@@ -145,7 +145,9 @@ const static struct {
     KeySym normal, dead;
 } dead_keys[] = {
     {XK_grave, XK_dead_grave},
+    {XK_apostrophe, XK_dead_acute},             /* US:"=" on a Czech keyboard */
     {XK_acute, XK_dead_acute},
+    {UKEYSYM (0x384), XK_dead_acute},           /* US:";" on a Greek keyboard */
     {XK_asciicircum, XK_dead_circumflex},
     {UKEYSYM (0x2c6), XK_dead_circumflex},	/* MODIFIER LETTER CIRCUMFLEX ACCENT */
     {XK_asciitilde, XK_dead_tilde},
@@ -572,7 +574,7 @@ Bool QuartzReadSystemKeymap(darwinKeyboardInfo *info) {
 #endif
     const void *chr_data = NULL;
     int num_keycodes = NUM_KEYCODES;
-    UInt32 keyboard_type = 0;
+    UInt32 keyboard_type = LMGetKbdType();
     int i, j;
     OSStatus err;
     KeySym *k;
@@ -580,7 +582,6 @@ Bool QuartzReadSystemKeymap(darwinKeyboardInfo *info) {
 
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
     TISInputSourceRef currentKeyLayoutRef = TISCopyCurrentKeyboardLayoutInputSource();
-    keyboard_type = LMGetKbdType();
 
     if (currentKeyLayoutRef) {
       currentKeyLayoutDataRef = (CFDataRef )TISGetInputSourceProperty(currentKeyLayoutRef, kTISPropertyUnicodeKeyLayoutData);
